/**
 * Copyright IRAP (CNRS/UPS/CNES)
 * 
 * The complete list of authors can be seen on CASSIS website at
 * the following URL "http://cassis.irap.omp.eu/?page=authors".
 * 
 * This software is a computer program whose purpose is to display, analyze
 * and create model of astrophysical spectra. It allow to use database and
 * related virtual observatory tools.
 * 
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software.  You can  use, 
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info". 
 * 
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability. 
 * 
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or 
 * data to be ensured and,  more generally, to use and operate it in the 
 * same conditions as regards security. 
 * 
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 */

package eu.omp.irap.cassis.gui.model.lineanalysis;

import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;

import eu.omp.irap.cassis.common.Template;
import eu.omp.irap.cassis.gui.plot.line.LineSpectrumView;
import eu.omp.irap.cassis.lineanalysis.LineAnalysisResult;
import eu.omp.irap.cassis.properties.Software;

public class LineSpectrumMain {

	public static void main(String[] args) {
		Software.initTemplateConfiguration();
		final LineSpectrumView lineSpectrumView = new LineSpectrumView();
		LineAnalysisPanel lineAnalysisPanel = new LineAnalysisPanel();
		lineAnalysisPanel.getControl().getModel().setTemplate(Template.DEFAULT_TEMPLATE);
		JFrame frame = new JFrame();
		JMenuBar jMenuBar = new JMenuBar();
		final JMenu c1 = new JMenu("File");
		final JMenu c = new JMenu("Config");
		c.add(new JMenuItem("Load"));
		c.add(new JMenuItem("Save"));
		jMenuBar.add(c1);
		jMenuBar.add(c);
		frame.setJMenuBar(jMenuBar);
		frame.setContentPane(lineAnalysisPanel);
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		frame.pack();
		frame.setVisible(true);
		lineAnalysisPanel.getControl().addLteLineTableListener(new LineAnalysisDisplayListener() {

			@Override
			public void lteLineTableDisplayClicked(LineAnalysisDisplayEvent e) {

				String telescope = e.getLteLineTableModel().getParametersModel().getSelectedTelescope();

				LineAnalysisResult lteLineResult = e.getLteLineResult();
				lteLineResult.setTelescope(telescope);
				lineSpectrumView.displayResult(lteLineResult);
				frame.setContentPane(lineSpectrumView);
				frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
				frame.pack();
				frame.setVisible(true);
			}
		});

	}

}
