/**
 * Copyright IRAP (CNRS/UPS/CNES)
 * 
 * The complete list of authors can be seen on CASSIS website at
 * the following URL "http://cassis.irap.omp.eu/?page=authors".
 * 
 * This software is a computer program whose purpose is to display, analyze
 * and create model of astrophysical spectra. It allow to use database and
 * related virtual observatory tools.
 * 
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software.  You can  use, 
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info". 
 * 
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability. 
 * 
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or 
 * data to be ensured and,  more generally, to use and operate it in the 
 * same conditions as regards security. 
 * 
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 */

package eu.omp.irap.cassis.gui.otherspecies;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.HashMap;
import java.util.Map;

import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JColorChooser;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.SwingUtilities;
import javax.swing.border.TitledBorder;

import eu.omp.irap.cassis.common.MoleculeDescription;
import eu.omp.irap.cassis.common.TypeFrequency;
import eu.omp.irap.cassis.common.axes.UNIT;
import eu.omp.irap.cassis.database.access.param.AijLineStrengthSelected;
import eu.omp.irap.cassis.gui.model.table.JCassisTable;
import eu.omp.irap.cassis.gui.plot.selection.SelectionPanel;
import eu.omp.irap.cassis.gui.template.JComboBoxTemplate;
import eu.omp.irap.cassis.gui.util.JDoubleCassisTextField;
import eu.omp.irap.cassis.gui.util.MouseMoleculeInfo;
import eu.omp.irap.cassis.properties.Software;

/**
 * @author glorian
 *
 */
public class SpeciesTab extends JPanel implements MoleculeSelectionnable{

	private static final String CANCEL_TEXT = "Cancel";
	private static final String DISPLAY_TEXT = "Display";
	private static final long serialVersionUID = 567387585486017805L;
	private JCheckBox speciesSignalBox;
	private JCheckBox speciesImageBox;
	
	private JComboBox<String> energyLabelComboBox;
	private JComboBox<UNIT> energyUnitComboBox;
	private final JLabel minEnergyLabel = new JLabel("min  :");
	private final JLabel maxEnergyLabel = new JLabel("max  :");
	private JDoubleCassisTextField minEnergyField;
	private JDoubleCassisTextField maxEnergyField;

	private JComboBox<String> weightLabelComboBox;
	private final JLabel minWeightLabel = new JLabel("min  :");
	private final JLabel maxWeightLabel = new JLabel("max  :");
	private JDoubleCassisTextField minWeightField;
	private JDoubleCassisTextField maxWeightField;
	
	private JComboBox<String> strengthLabelComboBox;
	private JComboBox<String> strengthUnitComboBox;
	
	private final JLabel minAijStrengthLabel = new JLabel("min  :");
	private final JLabel maxAijStrengthLabel = new JLabel("max  :");
	private JDoubleCassisTextField minAijField;
	private JDoubleCassisTextField maxAijField;

	private JComboBox<String> shiftWaveLabelComboBox;
	private final JComboBox<String> vlsrUnitCombobox = new JComboBox<>(new String[]{UNIT.KM_SEC_MOINS_1.toString()});
	private final JLabel offsetWaveDataLabel = new JLabel("data :");
	private final JLabel offsetWavePlotLabel = new JLabel("plot :");
	private JDoubleCassisTextField shiftWaveDataField;
	private JDoubleCassisTextField shiftWavePlotField;

	private JCassisTable<MoleculeDescription> moleculesTable;
	private JComboBoxTemplate comboBoxTemplate;
	private JButton displayButton;

	private JButton buttonColor;
	private JButton buttonColorImage;
	private SpeciesModel model;

	private JPanel thresholdPanel;
	private JPanel displayPanel;
	private JPanel speciesPanel;

	private JPanel energyPanel;
	private JPanel strengthPanel;
	private JPanel weightPanel;
	private JPanel shiftWavePanel;
	private SpeciesControl control;
	private SelectionPanel selectionPanel;
	private JPanel frequencyPanel;
	private JComboBox<String> frequencyComboBox;
	private JCheckBox limitVisibleDataCheckBox;
	private JLabel runningLabel;
	


	/**
	 * Constructor.
	 */
	public SpeciesTab() {
		this(new SpeciesControl(new SpeciesModel(true)));
	}

	public SpeciesTab(SpeciesControl control) {
		setControl(control);
		model = control.getModel();
		setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));

		add(getThresholdPanel());
		add(getSpeciesPanel());
		JPanel temp = new JPanel(new BorderLayout());
		temp.add(getSelectionPanel(), BorderLayout.NORTH);
		add(temp);
		add(getDisplayPanel());
		setSize(new Dimension(300, 500));
		getComboBoxTemplate().setSelectedItem(model.getTemplate());
//		model.getCassisTableMoleculeModel().selectAll();
	}

	private void setControl(SpeciesControl control) {
		this.control = control;
		control.setView(this);
	}

	private JPanel getSelectionPanel() {
		if (selectionPanel == null) {
			selectionPanel = new SelectionPanel();
			selectionPanel.setVisible(false);
		}
		return selectionPanel;
	}

	private JComponent getSpeciesPanel() {
		if (speciesPanel == null) {
			speciesPanel = new JPanel();
			speciesPanel.setLayout(new BorderLayout());
			speciesPanel.setBorder(new TitledBorder("Template"));

			speciesPanel.add(getComboBoxTemplate(), BorderLayout.NORTH);
			JScrollPane scrollPane = new JScrollPane(getMoleculesTable());
			speciesPanel.add(scrollPane, BorderLayout.CENTER);
			getMoleculesTable().setScrollPane(scrollPane);
		}
		return speciesPanel;
	}

	private JPanel getThresholdPanel() {
		if (thresholdPanel == null) {
			thresholdPanel = new JPanel();
			thresholdPanel.setLayout(new FlowLayout(FlowLayout.LEFT));
			thresholdPanel.setBorder(new TitledBorder("Thresholds and Settings"));
			JPanel temp = new JPanel();
			temp.setLayout(new GridBagLayout());
			GridBagConstraints c = new GridBagConstraints();
			c.fill = GridBagConstraints.HORIZONTAL;
			c.anchor = GridBagConstraints.LINE_START;
			c.gridx = 0;
			c.gridy = 0;

			temp.add(getEnergyPanel(), c);
			c.gridx = 0;
			c.gridy = 1;
			temp.add(getWeightPanel(), c);
			c.gridx = 0;
			c.gridy = 2;
			temp.add(getStrengthPanel(), c);
			c.gridx = 0;
			c.gridy = 3;
			temp.add(getShiftWavePanel(), c);
			c.gridx = 0;
			c.gridy = 4;
			temp.add(getFrequencyPanel(), c);
			thresholdPanel.add(temp);
		}
		return thresholdPanel;
	}

	private JPanel getFrequencyPanel() {
		if (frequencyPanel == null) {
			frequencyPanel = new JPanel();
			frequencyPanel.setBorder(new TitledBorder("Frequency Scale"));

			frequencyPanel.add(getFrequencyComboBox());
			frequencyPanel.setVisible(false);

		}
		return frequencyPanel;
	}

	public JComboBox<String> getFrequencyComboBox() {
		if (frequencyComboBox == null) {
			frequencyComboBox = new JComboBox<>(new String[]{TypeFrequency.REST.forSave(),
												 TypeFrequency.SKY.forSave()});
			frequencyComboBox.setEnabled(false);
			frequencyComboBox.setSelectedItem(model.getTypeFrequency());
			frequencyComboBox.addActionListener(e -> {
				TypeFrequency value = TypeFrequency.toFreq((String) ((JComboBox<?>)e.getSource()).getSelectedItem());
				model.setTypeFrequency(value);
			});
		}
		return frequencyComboBox;
	}

	private JPanel getShiftWavePanel() {
		if (shiftWavePanel == null) {
			shiftWavePanel = new JPanel();
			shiftWavePanel.setLayout(new FlowLayout(FlowLayout.LEFT));

			shiftWavePanel.add(getShiftWaveLabelComboBox());
			shiftWavePanel.add(offsetWaveDataLabel);
			shiftWavePanel.add(getVlsrDataField());
			shiftWavePanel.add(offsetWavePlotLabel);
			shiftWavePanel.add(getVlsrPlotField());
			shiftWavePanel.add(vlsrUnitCombobox);
		}
		return shiftWavePanel;
	}

	public JDoubleCassisTextField getVlsrPlotField() {
		if (shiftWavePlotField == null) {
			shiftWavePlotField = new JDoubleCassisTextField(4, 0.);
			shiftWavePlotField.addPropertyChangeListener("value",
					evt -> model.setVlsrPlot(Double.parseDouble(evt.getNewValue().toString())));
		}
		return shiftWavePlotField;
	}

	public JDoubleCassisTextField getVlsrDataField() {
		if (shiftWaveDataField == null) {
			shiftWaveDataField = new JDoubleCassisTextField(4, 0.);
			shiftWaveDataField.setEditable(false);
			shiftWaveDataField.addPropertyChangeListener("value",
											evt -> model.setVlsrData(Double.parseDouble(evt.getNewValue().toString())));
		}
		return shiftWaveDataField;
	}

	private JPanel getStrengthPanel() {
		if (strengthPanel == null) {
			strengthPanel = new JPanel();
			strengthPanel.setLayout(new FlowLayout(FlowLayout.LEFT));

			minAijStrengthLabel.setLabelFor(getMinStrengthField());
			maxAijStrengthLabel.setLabelFor(getMaxStrengthField());
			
			energyPanel.add(getEnergyLabelComboBox());
			energyPanel.add(minEnergyLabel);
			energyPanel.add(getMinEnergyField());
			energyPanel.add(maxEnergyLabel);
			energyPanel.add(getMaxEnergyField());
			energyPanel.add(getEnergyUnitComboBox());

			strengthPanel.add(getStrengthLabelComboBox());
			strengthPanel.add(minAijStrengthLabel);
			strengthPanel.add(getMinStrengthField());
			strengthPanel.add(maxAijStrengthLabel);
			strengthPanel.add(getMaxStrengthField());
			strengthPanel.add(getStrengthUnitComboBox());
		}
		return strengthPanel;
	}

	public JDoubleCassisTextField getMinStrengthField() {
		if (minAijField == null) {
			minAijField = new JDoubleCassisTextField(4, model.getAijMinLines());
			Map<String, Double> symbols = new HashMap<>();
			symbols.put("*", 0.0);
			minAijField.setSymbols(symbols);
			minAijField.addPropertyChangeListener("value",
					evt -> model.setAijMinLines(Double.parseDouble(evt.getNewValue().toString())));
		}
		return minAijField;
	}

	public JDoubleCassisTextField getMaxStrengthField() {
		if (maxAijField == null) {
			maxAijField = new JDoubleCassisTextField(4, model.getAijMaxLines());
			maxAijField.addPropertyChangeListener("value",
					evt -> model.setAijMaxLines(Double.parseDouble(evt.getNewValue().toString())));
		}
		return maxAijField;
	}

	private JPanel getEnergyPanel() {
		if (energyPanel == null) {
			energyPanel = new JPanel();
			energyPanel.setLayout(new FlowLayout(FlowLayout.LEFT));

			minEnergyLabel.setLabelFor(getMinEnergyField());
			maxEnergyLabel.setLabelFor(getMaxEnergyField());

			energyPanel.add(getEnergyLabelComboBox());
			energyPanel.add(minEnergyLabel);
			energyPanel.add(getMinEnergyField());
			energyPanel.add(maxEnergyLabel);
			energyPanel.add(getMaxEnergyField());
			energyPanel.add(getEnergyUnitComboBox());
		}
		return energyPanel;
	}

	public JComboBox<String>  getEnergyLabelComboBox() {
		if (energyLabelComboBox == null) {
			energyLabelComboBox= new JComboBox<>(new String[]{"eup", "elow"});
			energyLabelComboBox.addActionListener(e -> {
				boolean levelUpSelected = "eup".equals(energyLabelComboBox.getSelectedItem());
				model.setEnergyUpSelected(levelUpSelected);
			});
		}
		
		return energyLabelComboBox;
	}
	
	private JPanel getWeightPanel() {
		if (weightPanel == null) {
			weightPanel = new JPanel();
			weightPanel.setLayout(new FlowLayout(FlowLayout.LEFT));

			minWeightLabel.setLabelFor(getMinWeightField());
			maxWeightLabel.setLabelFor(getMaxWeightField());

			weightPanel.add(getWeightLabelComboBox());
			weightPanel.add(minWeightLabel);
			weightPanel.add(getMinWeightField());
			weightPanel.add(maxWeightLabel);
			weightPanel.add(getMaxWeightField());
		}
		return weightPanel;
	}

	public JComboBox<String>  getWeightLabelComboBox() {
		if (weightLabelComboBox == null) {
			weightLabelComboBox= new JComboBox<>(new String[]{"gup", "glow"});
			weightLabelComboBox.setEnabled(false);

			weightLabelComboBox.addActionListener(e -> {
				//TODO
//					boolean levelUpSelected = "gup".equals(momentLabelComboBox.getSelectedItem());
//					model.setMomentUpSelected(levelUpSelected);
			});
		}
		
		return weightLabelComboBox;
	}
	
	public JComboBox<String>  getStrengthLabelComboBox() {
		if (strengthLabelComboBox == null) {
			strengthLabelComboBox= new JComboBox<>(new String[]{"A","f"});
			strengthLabelComboBox.addActionListener(e -> {
				AijLineStrengthSelected aijLineStrengthSelected =

				"A".equals(strengthLabelComboBox.getSelectedItem())?
						AijLineStrengthSelected.AIJ:AijLineStrengthSelected.STRENGTH;

				model.setStrengthSelected(aijLineStrengthSelected);
				if (aijLineStrengthSelected.equals(AijLineStrengthSelected.AIJ)) {
					getStrengthUnitComboBox().setSelectedItem("s-1");
				} else {
					getStrengthUnitComboBox().setSelectedItem("Dieber");
				}
			});
		}
		
		return strengthLabelComboBox;
	}
	
	public JComboBox<String> getStrengthUnitComboBox() {
		if (strengthUnitComboBox == null) {
//			strengthUnitComboBox= new JComboBox<UNIT>(
//					new UNIT[]{UNIT.KELVIN, UNIT.CM_MINUS_1, UNIT.JOULE});
			strengthUnitComboBox= new JComboBox<>(
					new String[]{"s-1", "Dieber"});
			strengthUnitComboBox.setEnabled(false);
//			strengthUnitComboBox.addActionListener(new ActionListener() {
//				
//				@Override
//				public void actionPerformed(ActionEvent e) {
//					model.setEnergyUnitSelected((UNIT)(energyUnitComboBox.getSelectedItem()));
//					
//				}
//			});
		}
		return strengthUnitComboBox;
	}

	public JComboBox<String> getShiftWaveLabelComboBox() {
		if (shiftWaveLabelComboBox == null) {
			shiftWaveLabelComboBox = new JComboBox<>(new String[]{"Vlsr", "Redshift"});
			shiftWaveLabelComboBox.addActionListener(
				new ActionListener() {
					@Override
				    public void actionPerformed(ActionEvent actionEvent) {
						vlsrUnitCombobox.setVisible(shiftWaveLabelComboBox.getSelectedItem().equals("Vlsr"));
						model.setShiftWaveType(shiftWaveLabelComboBox.getSelectedItem().toString());
					}
				}
			);
		}
		return shiftWaveLabelComboBox;
	}

	public JComboBox<UNIT> getEnergyUnitComboBox() {
		if (energyUnitComboBox == null) {
			energyUnitComboBox= new JComboBox<>(
					new UNIT[]{UNIT.KELVIN, UNIT.CM_MINUS_1, UNIT.JOULE});
			energyUnitComboBox.addActionListener(
					e -> model.setEnergyUnitSelected((UNIT)(energyUnitComboBox.getSelectedItem())));
		}
		return energyUnitComboBox;
	}

	public JDoubleCassisTextField getMaxEnergyField() {
		if (maxEnergyField == null) {
			maxEnergyField = new JDoubleCassisTextField(4,model.getEupMaxLines());
			maxEnergyField.addPropertyChangeListener("value", evt -> {
				if(model.isEnergyUpSelected()) {
					model.setEupMaxSpecies(Double.parseDouble(evt.getNewValue().toString()));
				} else {
					model.setElowMaxSpecies(Double.parseDouble(evt.getNewValue().toString()));
				}
			});
		}
		return maxEnergyField;
	}

	public JDoubleCassisTextField getMinEnergyField() {
		if (minEnergyField == null) {
			minEnergyField = new JDoubleCassisTextField(4,model.getEupMinLines());
			Map<String, Double> symbols = new HashMap<>();
			symbols.put("*", 0.0);
			minEnergyField.setSymbols(symbols);
			minEnergyField.addPropertyChangeListener("value", evt -> {
				if (model.isEnergyUpSelected()) {
					model.setEupMinSpecies(Double.parseDouble(evt.getNewValue().toString()));
				} else {
					model.setElowMinSpecies(Double.parseDouble(evt.getNewValue().toString()));
				}
			});
		}
		return minEnergyField;
	}

	public JDoubleCassisTextField getMaxWeightField() {
		if (maxWeightField == null) {
			maxWeightField = new JDoubleCassisTextField(4,model.getEupMaxLines());
			maxWeightField.addPropertyChangeListener("value", evt -> {
				//TODO
//					if(model.isMomentUpSelected()) {
//						model.setEupMaxSpecies(Double.valueOf(evt.getNewValue().toString()));
//					} else {
//						model.setElowMaxSpecies(Double.valueOf(evt.getNewValue().toString()));
//					}
			});
		}
		return maxWeightField;
	}

	public JDoubleCassisTextField getMinWeightField() {
		if (minWeightField == null) {
			minWeightField = new JDoubleCassisTextField(4,model.getEupMinLines());
			Map<String, Double> symbols = new HashMap<>();
			symbols.put("*", 0.0);
			minWeightField.setSymbols(symbols);
			minWeightField.addPropertyChangeListener("value", evt -> {
				//TODO
//					if (model.isMomentUpSelected()) {
//						model.setEupMinSpecies(Double.valueOf(evt.getNewValue().toString()));
//					} else {
//						model.setElowMinSpecies(Double.valueOf(evt.getNewValue().toString()));
//					}
			});
		}
		return minWeightField;
	}
	
	private JPanel getDisplayPanel() {
		if (displayPanel == null) {
			displayPanel = new JPanel();
			JPanel temp;
			temp = new JPanel();
			temp.setLayout(new GridBagLayout());

			GridBagConstraints c = new GridBagConstraints();
			c.fill = GridBagConstraints.HORIZONTAL;
			c.anchor = GridBagConstraints.LINE_START;
			c.gridx = 0;
			c.gridy = 0;
			temp.add(getOtherSpeciesBox(), c);
			c.gridx = 1;
			c.gridy = 0;
			temp.add(getButtonColor(), c);
			c.gridx = 0;
			c.gridy = 1;
			temp.add(getOtherSpeciesImageBox(), c);
			c.gridx = 1;
			c.gridy = 1;
			temp.add(getButtonColorImage(), c);
			c.gridx = 0;
			c.gridy = 2;
			if (model.isDisplayVisibleDataOption()) {
				c.gridwidth = 3;
				temp.add(getLimitVisibleDataCheckBox(), c);
				c.gridwidth = 1;
				c.gridy = 3;
			}
			temp.add(getDisplayButton(), c);
			c.gridx = 2;
			temp.add(getRunningLabel(), c);
			displayPanel.setLayout(new FlowLayout(FlowLayout.LEFT));
			displayPanel.setBorder(new TitledBorder("Display"));
			displayPanel.add(temp);
		}
		return displayPanel;
	}

	public JCheckBox getOtherSpeciesImageBox() {
		if (speciesImageBox == null) {
			speciesImageBox = new JCheckBox("Show image");
			speciesImageBox.setSelected(false);
			speciesImageBox.addActionListener(e -> model.setSpeciesImageVisible(speciesImageBox.isSelected()));
		}
		return speciesImageBox;
	}

	public JCheckBox getOtherSpeciesBox() {
		if (speciesSignalBox == null) {
			speciesSignalBox = new JCheckBox("Show signal");
			speciesSignalBox.setSelected(false);
			speciesSignalBox.addActionListener(e -> model.setSpeciesSignalVisible(speciesSignalBox.isSelected()));
		}
		return speciesSignalBox;
	}

	public JButton getButtonColor() {
		if (buttonColor == null) {
			buttonColor = new JButton();
			buttonColor.setBackground(model.getColorSpeciesSignal());
			buttonColor.setSize(43, 50);
			buttonColor.addActionListener(e -> {
				Color newColor = JColorChooser.showDialog(SpeciesTab.this,
						"Choose Background Color",
						buttonColor.getBackground());
				if (newColor != null) {
					model.setColorSpeciesSignal(newColor);
				}
			});
		}
		return buttonColor;
	}

	public JButton getButtonColorImage() {
		if (buttonColorImage == null) {
			buttonColorImage = new JButton();
			buttonColorImage.setBackground(model.getColorSpeciesImage());
			buttonColorImage.setEnabled(true);
			buttonColorImage.addActionListener(e -> {
				Color newColor = JColorChooser.showDialog(SpeciesTab.this, "Choose Background Color",
															buttonColorImage.getBackground());
				if (newColor != null) {
					model.setColorSpeciesImage(newColor);
				}
			});
		}
		return buttonColorImage;
	}

	public JButton getDisplayButton() {
		if (displayButton == null) {
			displayButton = new JButton(DISPLAY_TEXT);
			displayButton.addActionListener(e -> control.fireSpeciesDisplayClicked());
		}
		return displayButton;
	}

	public void setShowSpeciesImageEnabled(boolean value) {
		if (!value) {
			getOtherSpeciesBox().setText("Show");
		}
		else {
			getOtherSpeciesBox().setText("Show signal");
		}
		getOtherSpeciesImageBox().setVisible(value);
		buttonColorImage.setVisible(value);
		model.setHaveImage(value);
	}

	public boolean isSpeciesImageEnabled() {
		return getOtherSpeciesImageBox().isVisible();
	}

	/**
	 * Set the selection of Other species.
	 *
	 * @param value The selection value.
	 */
	public void selectSpeciesSignal(boolean value) {
		getOtherSpeciesBox().setSelected(value);
	}

	public void selectSpeciesImage(boolean value) {
		getOtherSpeciesImageBox().setSelected(value);
	}

	public SpeciesModel getModel() {
		return model;
	}

	public void setSpeciesSignalVisible(boolean enabled) {
		model.setSpeciesSignalVisible(enabled);
	}

	public void setSpeciesImageVisible(boolean enabled) {
		model.setSpeciesImageVisible(enabled);
	}

	public void setOtherDisplayButtonEnabled(boolean val) {
		displayButton.setEnabled(val);
	}

	public void initVlsr(double d) {
		shiftWaveDataField.setValue(d);
		shiftWavePlotField.setValue(d);
	}

	public SpeciesControl getControl() {
		return control;
	}

	public JComboBoxTemplate getComboBoxTemplate() {
		if (comboBoxTemplate == null) {
			comboBoxTemplate = new JComboBoxTemplate(false, true);
			comboBoxTemplate.setSelectedItem(model.getTemplate());
			comboBoxTemplate.addActionListener(e -> {
				String template = (String) comboBoxTemplate.getSelectedItem();
				if (template != null && !JComboBoxTemplate.OPERATIONS_TEMPLATE.equals(template) &&
										!JComboBoxTemplate.TEMPLATE.equals(template)) {
					model.setTemplate(template);
				}
			});

		}
		return comboBoxTemplate;
	}

	public JCassisTable<MoleculeDescription> getMoleculesTable() {
		if (moleculesTable == null) {
			moleculesTable = new JOtherSpeciesTable<>(
					model.getCassisTableMoleculeModel(), false, true, this);
			moleculesTable.setTemplateCombo(getComboBoxTemplate());
			moleculesTable.getColumn("Name").setMinWidth(130);
			moleculesTable.getColumn("Id").setMinWidth(20);
//			moleculesTable.getColumn("Id").setMaxWidth(52);
//			moleculesTable.getColumn("Id").setPreferredWidth(90);
			moleculesTable.getColumn("Database").setMinWidth(60);
//			moleculesTable.getColumn("Database").setMaxWidth(66);
//			moleculesTable.getColumn("Database").setPreferredWidth(66);
			moleculesTable.getColumn("Sel.").setMinWidth(25);
			moleculesTable.getColumn("Sel.").setMaxWidth(25);
			moleculesTable.getColumn("Sel.").setPreferredWidth(25);
			moleculesTable.getSelectionModel().addListSelectionListener(
					event -> moleculesTable.getSelectionModel().clearSelection());
			moleculesTable.addMouseListener(new MouseAdapter() {
				@Override
				public void mouseClicked(MouseEvent e) {
					if (Software.getUserConfiguration().isAutoOtherSpeciesDisplayed() && SwingUtilities.isLeftMouseButton(e)) {
						getDisplayButton().doClick();
					}
				}
			});
			MouseMoleculeInfo.decorate(moleculesTable);
		}
		return moleculesTable;
	}

	/**
	 * Create if needed then return the limit visible data JCheckBox.
	 *
	 * @return the limit visible data JCheckBox.
	 */
	public JCheckBox getLimitVisibleDataCheckBox() {
		if (limitVisibleDataCheckBox == null) {
			limitVisibleDataCheckBox = new JCheckBox("Limit research to visible data");
			limitVisibleDataCheckBox.setSelected(model.isLimitVisibleData());
			limitVisibleDataCheckBox.addActionListener(
					e -> model.setLimitVisibleData(limitVisibleDataCheckBox.isSelected()));
		}
		return limitVisibleDataCheckBox;
	}

	@Override
	public void moleculeSelectedbyList() {
		if (Software.getUserConfiguration().isAutoOtherSpeciesDisplayed()) {
			getDisplayButton().doClick();
		}
	}

	@Override
	public void headerAllComputeClicked() {
		if (Software.getUserConfiguration().isAutoOtherSpeciesDisplayed()) {
			getDisplayButton().doClick();
		}
	}

	/**
	 * Create if needed then return the "Running..." label.
	 *
	 * @return the "Running..." label.
	 */
	public JLabel getRunningLabel() {
		if (runningLabel == null) {
			runningLabel = new JLabel("Running...");
			runningLabel.setVisible(false);
		}
		return runningLabel;
	}

	/**
	 * Mark the panel as running an OS computation.
	 */
	public void setRunning() {
		getRunningLabel().setVisible(true);
		getDisplayButton().setText(CANCEL_TEXT);
	}

	/**
	 * Mark the panel as not running an OS computation.
	 */
	public void setDone() {
		getRunningLabel().setVisible(false);
		getDisplayButton().setText(DISPLAY_TEXT);
	}
	
	public static void main(String[] args) {
		Software.initTemplateConfiguration();
		JPanel panel = new SpeciesTab();
		JFrame frame = new JFrame("Test other species");
		frame.setContentPane(panel);
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		frame.setVisible(true);
		frame.pack();
	}
}
